use crate::assets::Assets;
use crate::config::{SCREEN_HEIGHT, SCREEN_WIDTH};
use macroquad::color;
use macroquad::color::Color;
use macroquad::math::Vec2;
use macroquad::prelude::{draw_texture, draw_texture_ex, get_frame_time, DrawTextureParams};
use macroquad::rand::RandomRange;

pub struct Sky {
    sun: Sun,
    clouds: Vec<Cloud>,
}

impl Default for Sky {
    fn default() -> Self {
        let sun = Sun {
            rot: -core::f32::consts::FRAC_PI_6,
            pos: Vec2::new(25.0, 25.0),
        };

        let mut clouds = Vec::new();
        const CLOUD_CNT: usize = 20;

        for i in 0..CLOUD_CNT {
            let pos_x = f32::gen_range(0.0, SCREEN_WIDTH);
            let pos_y = f32::gen_range(0.0, 16384.0);
            let speed = f32::gen_range(5.0, 15.0);
            clouds.push(Cloud {
                pos: Vec2::new(pos_x, pos_y),
                speed,
                shade: 0.75 + 0.25 * i as f32 / CLOUD_CNT as f32,
            })
        }

        Self { sun, clouds }
    }
}

impl Sky {
    pub fn render(&mut self, assets: &Assets) {
        self.sun.draw(assets);

        self.clouds.iter_mut().for_each(|cloud| cloud.draw(assets));
    }
}

pub struct Sun {
    pub pos: Vec2,
    pub rot: f32,
}

impl Sun {
    pub fn draw(&self, assets: &Assets) {
        draw_texture_ex(
            assets.sun_texture,
            self.pos.x,
            self.pos.y,
            color::WHITE,
            DrawTextureParams {
                rotation: self.rot,
                ..Default::default()
            },
        );
    }
}

#[derive(Debug)]
pub struct Cloud {
    pos: Vec2,
    speed: f32,
    shade: f32,
}

impl Cloud {
    pub fn draw(&mut self, assets: &Assets) {
        self.pos.x -= self.speed * get_frame_time();

        let w = assets.cloud_texture.width();
        let h = assets.cloud_texture.height();

        draw_texture(
            assets.cloud_texture,
            self.pos.x.rem_euclid(SCREEN_WIDTH + w * 4.0) - w * 2.0,
            self.pos.y.rem_euclid(SCREEN_HEIGHT / 3.0 - h),
            Color::new(self.shade, self.shade, 1.0, 1.0),
        );
    }
}
